@@ -1221,6 +1221,7 @@ extern struct list_head super_blocks;
 extern spinlock_t sb_lock;
 
 /* sb->s_iflags */
+#define SB_I_NOEXEC	0x00000002	/* Ignore executables on this fs */
 #define SB_I_NOSUID	0x00000004	/* Ignore suid on this fs */
 
 /* Possible states of 'frozen' field */
@@ -2831,5 +2832,6 @@ static inline bool dir_relax(struct inode *inode)
 }
 
 extern bool path_nosuid(const struct path *path);
+extern bool path_noexec(const struct path *path);
 
 #endif /* _LINUX_FS_H */
